(:~
 : This modules handles calling the right HTML file for each selected file of the
 : edition view. It is called by book.html.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 :)

xquery version "3.1";

module namespace edition="http://bdn-edition.de/ns/edition";

declare namespace templates="http://exist-db.org/xquery/templates";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace index-query="https://mollenhauer-edition.de/ns/index-query" at "./index/index-query.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "kmg-util.xqm";
import module namespace editorial="https://mollenhauer-edition.de/ns/editorial" at "editorial-content/editorial.xqm";

import module namespace console="http://exist-db.org/xquery/console";

declare default element namespace "http://www.w3.org/1999/xhtml";

declare variable $edition:HTML-path := $config:edition;

(:~
 : The main function sets several parameters which are necessary for the displ
 : of the text, e.g. the start and ending chapters which users can set manually
 : afterwards.
 :
 : Sample URL: http://localhost:8080/exist/apps/sade/book.html?file=12345&edition=app
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @version 1.0
 : @return a map object with data taht is important for edition:display
 :)
declare
  %templates:wrap
  function edition:main($node as node(), $model as map(*)) as map(*)
{
  map
  {
    "file": request:get-parameter("file", ""),
    "start-chapter": request:get-parameter("startChapter", "1"),
    "end-chapter": request:get-parameter("endChapter", "9999"),
    "edition": request:get-parameter("edition", "digital")
  }
};

(:~
 : Selects and displays the appropriate HTML site according to the map data that
 : has been created in edition:main.
 :
 : This function is called by bdn-utils:inject-cols.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @version 1.0
 : @return A xhtml:div which is embedded into book.html
 :)
declare
  %templates:wrap
  function edition:display($node as node()*, $model as map(*))
  as element(div)
{
  let $file := $model("file")
  let $mode := $model("edition")
  let $html-suffix :=
    switch ($mode)
      case "code" return
        "code"
      case "digital" return
        "digital"
      default return
        $mode
  let $html-filename := $file || "-" || $html-suffix || ".html"
  (:~ let $m := console:log($html-filename) ~:)
  let $doc := edition:open-file($html-filename, "edition")
  return
    $doc/article
};


(:~
 : Selects and displays the appropriate HTML site that holds the rdg spans
 : according to the map data that has been created in edition:main.
 :
 : This function is called by bdn-utils:inject-cols.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @version 1.0
 : @return A xhtml:div which is embedded into book.html
 :)
declare
  %templates:wrap
  function edition:display-variants-and-quotes($node as node()*, $model as map(*))
  as element(span)+
{
  let $file := $model("file")
  let $html-filename := $file || "-digital-variants-and-quotes.html"
  let $doc := edition:open-file($html-filename, "aux")

  return
    $doc/div/*
};

declare
  %templates:wrap
function edition:display-editorials($node as node()*, $model as map(*))
as element(div)*
{
  let $file := $model("file")
  let $edition := $model("edition")
  let $html-filename := $file || "-" || $edition || "-editorials.html"
  let $doc := edition:open-file($html-filename, "aux")
  let $non-entities := $doc/div/*[
    not(matches(@class, "entity-name"))
    and not(matches(@class, "note"))
    and not(matches(@class, "correction"))
    and not(matches(@class, "abbreviation"))
    and not(matches(@class, "addition"))
    and not(matches(@class, "addhighlight"))
    and not(matches(@class, "bibliographic-reference"))
    and not(matches(@class, "internal-reference"))
    and not(matches(@class, "musical-work"))
    ]
  let $corrections := $doc/div/*[matches(@class, "correction")]
  let $abbreviations := $doc/div/*[matches(@class, "abbreviation")]
  let $additions := $doc/div/*[matches(@class, "addition")]
  let $addhighlights := $doc/div/*[matches(@class, "addhighlight")]
  let $notes := $doc/div/*[matches(@class, "note")]
  let $entities := $doc/div/*[matches(@class, "entity-name")]
  let $bibliographic-references := $doc/div/*[matches(@class, "bibliographic-reference")]
  let $internal-references := $doc/div/*[matches(@class, "internal-reference")]
  let $musical-works := $doc/div/*[matches(@class, "musical-work")]

  return
    ($non-entities,
    for $correction in $corrections return
      editorial:correction($correction),
    for $addition in $additions return
      editorial:addition($addition, "addition"),
    for $addhighlight in $addhighlights return
      editorial:addition($addhighlight, "highlight"),
    for $abbreviation in $abbreviations return
      editorial:abbreviation($abbreviation),
    for $note in $notes return
      editorial:note($note),
    for $reference in $bibliographic-references return
      editorial:bibliographic-reference($reference),
    for $reference in $internal-references return
      editorial:internal-bibliographic-reference($reference),
    for $work in $musical-works return
      editorial:musical-work($work),
    for $entity in $entities return
      if ($entity/@data-ref and string-length($entity/@data-ref) > 0) then
        let $content := index-query:build-info($entity/@data-ref)
        return
          $content
        else
          $entity
    )
};

(:~
 : Displays the foreword.
 :)
declare
  %templates:wrap
  function edition:foreword($node as node()*, $model as map(*))
  as element(div)
{
  let $author := request:get-parameter("author", "")
  let $html-filename := $author || "-foreword.html"
  return
      doc($config:edition || $author || "/" || $html-filename)/*
};

(:~
 : Opens a given HTML file.
 :
 :)
declare function edition:open-file($html-filename as xs:string, $mode as xs:string) as node()*
{
    let $first-part := substring-before($html-filename, "-")
    let $author :=
        if (matches($first-part, "[0-9]")) then
            false()
        else
            true()
    return
      if($mode = "edition") then
        doc($config:edition || $html-filename)
      else
        doc($config:edition-aux || $html-filename)
};
